@@ -13,6 +13,7 @@
 
 #include <glib-object.h>
 #include <glib/gi18n.h>
+#include <glib/gprintf.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include "process-tree-model.h"
@@ -368,7 +369,8 @@ cb_send_signal (GtkMenuItem *mi, gpointer user_data)
 		dialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
 			(xtm_signal == XTM_SIGNAL_TERMINATE) ? _("Terminate task") : _("Kill task"));
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-			_("Are you sure you want to send a signal to the PID %d?"), pid);
+			_("Are you sure you want to send the %s signal to the PID %d?"),
+			(xtm_signal == XTM_SIGNAL_TERMINATE) ? _("terminate") : _("kill"), pid);
 		gtk_window_set_title (GTK_WINDOW (dialog), _("Task Manager"));
 		gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
 		res = gtk_dialog_run (GTK_DIALOG (dialog));
@@ -556,25 +558,40 @@ treeview_key_pressed (XtmProcessTreeView *treeview, GdkEventKey *event)
 {
 	guint pid;
 
-	if (event->keyval != GDK_KEY_Menu)
+	GtkTreeModel *model;
+	GtkTreeSelection *selection;
+	GtkTreeIter iter;
+	GdkModifierType modifiers;
+
+	modifiers = gtk_accelerator_get_default_mod_mask ();
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
+
+	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
 		return FALSE;
 
+	gtk_tree_model_get (model, &iter, XTM_PTV_COLUMN_PID, &pid, -1);
+
+	if (event->keyval == GDK_KEY_Menu)
 	{
-		GtkTreeModel *model;
-		GtkTreeSelection *selection;
-		GtkTreeIter iter;
-
-		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
-
-		if (!gtk_tree_selection_get_selected (selection, &model, &iter))
-			return FALSE;
-
-		gtk_tree_model_get (model, &iter, XTM_PTV_COLUMN_PID, &pid, -1);
+		popup_menu (treeview, pid, event->time, FALSE);
+		return TRUE;
+	}
+	else if (event->keyval == GDK_KEY_Delete)
+	{
+		/* Fake menuitem for the cb_send_signal callback */
+		GtkWidget *mi;
+		mi = gtk_menu_item_new_with_label (_("Stop"));
+		g_object_set_data (G_OBJECT (mi), "pid", GUINT_TO_POINTER (pid));
+		if ((event->state & modifiers) == GDK_SHIFT_MASK)
+			cb_send_signal (GTK_MENU_ITEM (mi), GINT_TO_POINTER (XTM_SIGNAL_KILL));
+		else
+			cb_send_signal (GTK_MENU_ITEM (mi), GINT_TO_POINTER (XTM_SIGNAL_TERMINATE));
+		return TRUE;
 	}
 
-	popup_menu (treeview, pid, event->time, FALSE);
-
-	return TRUE;
+	else
+		return FALSE;
 }
 
 static void
@@ -754,3 +771,56 @@ xtm_process_tree_view_get_model (XtmProcessTreeView *treeview)
 	return GTK_TREE_MODEL (treeview->model);
 }
 
+void
+xtm_process_tree_view_highlight_pid (XtmProcessTreeView *treeview, guint pid) {
+	GtkTreeModel	*model;
+	GtkTreePath		*path;
+	GtkTreeIter 	 iter;
+	gboolean     	 valid;
+	gboolean			 tree;
+	guint					 pid_iter;
+	GtkTreeIter	child_iter;
+	gboolean validParent;
+
+	g_object_get (treeview->settings, "process-tree", &tree, NULL);
+	model = GTK_TREE_MODEL (tree ? treeview->model_tree : treeview->model_filter);
+
+	g_return_if_fail (model != NULL);
+
+	/* Get first row in list store */
+	valid = gtk_tree_model_get_iter_first (model, &iter);
+
+	while (valid)
+	{
+		gtk_tree_model_get (model, &iter, XTM_PTV_COLUMN_PID, &pid_iter, -1);
+		if (pid == pid_iter)
+		{
+			path = gtk_tree_model_get_path (model, &iter);
+			gtk_tree_selection_select_path (gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview)), path);
+			gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (treeview), path, NULL, TRUE, 0.5, 0);
+			gtk_tree_path_free (path);
+			break;
+		}
+
+		if (gtk_tree_model_iter_has_child (model, &iter))
+		{
+			GtkTreeIter	parent_iter = iter;
+
+			valid = gtk_tree_model_iter_children (model, &iter, &parent_iter);
+		}
+		else
+		{
+			child_iter = iter;
+			valid = gtk_tree_model_iter_next (model, &iter);
+			if (tree && !valid)
+			{
+				//finding my way up again
+				do {
+					validParent = gtk_tree_model_iter_parent (model, &iter, &child_iter);
+					child_iter = iter;
+					valid = gtk_tree_model_iter_next (model, &iter);
+				} while (!valid && validParent);
+			}
+		}
+	}
+}
